static const char svg_tb_help[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_help.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"1.8731585\"     inkscape:cx=\"146.27347\"     inkscape:cy=\"132.45124\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <circle       style=\"fill:#000084;fill-opacity:1;stroke:none;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"path837\"       cx=\"33.866665\"       cy=\"263.13333\"       r=\"31.220833\" />    <g       aria-label=\"?\"       style=\"font-style:normal;font-weight:normal;font-size:10.58333302px;line-height:1.25;font-family:sans-serif;letter-spacing:0px;word-spacing:0px;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:0.26458332\"       id=\"text816\">      <path         d=\"m 36.843229,269.48333 h -8.954492 v -1.21543 q 0,-2.03398 0.818555,-3.59668 0.818555,-1.5875 3.447852,-4.01836 l 1.5875,-1.43867 q 1.413867,-1.28984 2.058789,-2.43086 0.669726,-1.14101 0.669726,-2.28203 0,-1.73633 -1.190625,-2.70371 -1.190625,-0.99219 -3.323828,-0.99219 -2.00918,0 -4.34082,0.84336 -2.331641,0.81856 -4.861719,2.45567 v -7.78868 q 3.001367,-1.04179 5.481836,-1.53789 2.480469,-0.49609 4.787305,-0.49609 6.052343,0 9.227343,2.48047 3.175,2.45566 3.175,7.19336 0,2.43086 -0.967383,4.36562 -0.967382,1.90996 -3.299023,4.11758 l -1.5875,1.41387 q -1.686719,1.53789 -2.207617,2.48047 -0.520899,0.91777 -0.520899,2.03398 z m -8.954492,3.6711 h 8.954492 v 8.83046 h -8.954492 z\"         style=\"font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;font-size:50.79999924px;font-family:sans-serif;-inkscape-font-specification:'sans-serif, Bold';font-variant-ligatures:normal;font-variant-caps:normal;font-variant-numeric:normal;font-feature-settings:normal;text-align:start;writing-mode:lr-tb;text-anchor:start;fill:#ffffff;fill-opacity:1;stroke-width:0.26458332\"         id=\"path1460\"         inkscape:connector-curvature=\"0\" />    </g>  </g></svg>";