static const char svg_default[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"default.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"0.97325205\"     inkscape:cx=\"-106.5448\"     inkscape:cy=\"61.325126\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title />      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <g       id=\"g4553\">      <rect         y=\"229.26665\"         x=\"0\"         height=\"67.73333\"         width=\"67.73333\"         id=\"rect895\"         style=\"fill:#000000;fill-opacity:1;stroke:none;stroke-width:1.56633496;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <rect         y=\"233.2354\"         x=\"4.2333331\"         height=\"59.53125\"         width=\"59.53125\"         id=\"rect895-9\"         style=\"display:inline;fill:#000084;fill-opacity:1;stroke:none;stroke-width:1.37666154;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <rect         y=\"237.20415\"         x=\"8.4666662\"         height=\"51.329166\"         width=\"51.329166\"         id=\"rect895-9-2\"         style=\"display:inline;fill:#0000ff;fill-opacity:1;stroke:none;stroke-width:1.18698812;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <rect         y=\"241.1729\"         x=\"12.7\"         height=\"43.127083\"         width=\"43.127083\"         id=\"rect895-9-2-0\"         style=\"display:inline;fill:#0042ff;fill-opacity:1;stroke:none;stroke-width:0.99731481;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <rect         y=\"245.40623\"         x=\"16.933332\"         height=\"34.660416\"         width=\"34.660416\"         id=\"rect895-9-2-0-6\"         style=\"display:inline;fill:#0084ff;fill-opacity:1;stroke:none;stroke-width:0.80152291;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <rect         y=\"249.37498\"         x=\"21.166666\"         height=\"26.458334\"         width=\"26.458334\"         id=\"rect895-9-2-0-6-8\"         style=\"display:inline;fill:#00c6ff;fill-opacity:1;stroke:none;stroke-width:0.61184955;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <rect         y=\"253.34373\"         x=\"25.4\"         height=\"18.25625\"         width=\"18.25625\"         id=\"rect895-9-2-0-6-8-9\"         style=\"display:inline;fill:#00ffff;fill-opacity:1;stroke:none;stroke-width:0.42217615;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <rect         y=\"257.31247\"         x=\"29.633333\"         height=\"10.054167\"         width=\"10.054167\"         id=\"rect895-9-2-0-6-8-9-2\"         style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:0.23250279;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />    </g>  </g></svg>";