static const char svg_tb_edit_shape_del_point[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_edit_shape_del_point.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"1.8761836\"     inkscape:cx=\"48.363289\"     inkscape:cy=\"132.81858\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <circle       style=\"fill:#ffffff;fill-opacity:1;stroke:#000000;stroke-width:3.70416641;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"path945\"       cx=\"32.808334\"       cy=\"238.79166\"       r=\"4.4979167\" />    <circle       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#000000;stroke-width:3.70416641;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"path945-3\"       cx=\"24.870832\"       cy=\"280.59583\"       r=\"4.4979167\" />    <circle       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#000000;stroke-width:3.70416641;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"path945-1\"       cx=\"43.920834\"       cy=\"288.53333\"       r=\"4.4979167\" />    <circle       style=\"display:inline;fill:#00c6ff;fill-opacity:1;stroke:#0042ff;stroke-width:3.70416641;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"path945-4\"       cx=\"57.150002\"       cy=\"257.84164\"       r=\"4.4979167\" />    <g       id=\"g977\">      <path         sodipodi:nodetypes=\"caszsc\"         inkscape:connector-curvature=\"0\"         id=\"path897\"         d=\"m 60.855193,238.26184 c 0,0 -23.48328,14.36585 -33.779906,23.39081 -7.714833,6.76203 -21.6104085,16.67439 -20.6780817,22.78254 0.3164417,2.07317 1.1268802,7.5604 5.5352037,5.92009 4.408324,-1.6403 15.092405,-20.24337 22.96496,-28.19785 11.89575,-12.01955 25.957824,-23.89559 25.957824,-23.89559 z\"         style=\"fill:#000084;fill-opacity:1;stroke:#000084;stroke-width:2.11666679;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <path         sodipodi:nodetypes=\"caszsc\"         inkscape:connector-curvature=\"0\"         id=\"path897-1\"         d=\"m 53.7407,281.90791 c 0,0 -10.404732,-17.00609 -17.047896,-24.36163 -7.685959,-8.51015 -19.929721,-23.41563 -26.03788,-22.48331 -2.073167,0.31645 -5.6505263,4.37367 -4.0102185,8.78199 1.6403008,4.40833 11.7209405,6.49741 20.4133745,13.5467 13.365016,10.8386 26.68262,24.51625 26.68262,24.51625 z\"         style=\"display:inline;fill:#000084;fill-opacity:1;stroke:#000084;stroke-width:2.11666656;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <ellipse         transform=\"rotate(-27.50411)\"         ry=\"1.5875\"         rx=\"1.3229166\"         cy=\"280.03345\"         cx=\"-81.897614\"         id=\"path920\"         style=\"fill:#000084;fill-opacity:1;stroke:none;stroke-width:2.11666656;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />    </g>  </g></svg>";