static const char svg_wksp_module_manager[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"wksp_module_manager.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"1.4176798\"     inkscape:cx=\"147.5815\"     inkscape:cy=\"169.96315\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\">    <rect       style=\"fill:#0042ff;fill-opacity:1;stroke:#000000;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect895\"       width=\"15.875001\"       height=\"55.827084\"       x=\"3.96875\"       y=\"237.20415\" />    <rect       style=\"fill:#ffffff;fill-opacity:1;stroke:#000000;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect895-6\"       width=\"10.583334\"       height=\"48.947918\"       x=\"19.84375\"       y=\"244.08331\" />    <rect       style=\"display:inline;fill:#0000ff;fill-opacity:1;stroke:#000000;stroke-width:2.64583302;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect895-6-8\"       width=\"7.9375014\"       height=\"59.795834\"       x=\"55.827084\"       y=\"233.2354\" />    <rect       style=\"display:inline;fill:#00ffff;fill-opacity:1;stroke:#000000;stroke-width:2.68015432;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect895-6-8-8\"       width=\"12.170834\"       height=\"58.099304\"       x=\"97.617378\"       y=\"242.60088\"       transform=\"matrix(1,0,-0.22415845,0.97455271,0,0)\" />    <rect       style=\"fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:2.64583302;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect979\"       width=\"5.2916665\"       height=\"5.2916665\"       x=\"9.260417\"       y=\"243.02501\" />    <rect       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:2.64583302;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect979-1\"       width=\"5.2916665\"       height=\"5.2916665\"       x=\"9.260417\"       y=\"253.41835\" />    <rect       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:2.64583302;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect979-4\"       width=\"5.2916665\"       height=\"5.2916665\"       x=\"9.260417\"       y=\"282.71249\" />    <rect       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:2.64583302;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect979-4-9\"       width=\"5.2916665\"       height=\"15.875\"       x=\"9.260417\"       y=\"264.45789\" />  </g></svg>";