static const char svg_tb_layout_print[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_layout_print.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"1.1124799\"     inkscape:cx=\"-78.637796\"     inkscape:cy=\"149.09588\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <path       inkscape:connector-curvature=\"0\"       d=\"M 53.869168,251.72688 H 13.864166 c -4.7434463,0 -8.5724993,3.82115 -8.5724993,8.55486 v 17.10972 H 16.721662 v 11.40644 h 34.289999 v -11.40644 h 11.430006 v -17.10972 c 0,-4.73371 -3.829053,-8.55486 -8.572499,-8.55486 z m -8.5725,31.36774 H 22.436666 V 268.8366 h 22.860002 z m 8.5725,-19.9613 c -1.571627,0 -2.857507,-1.28321 -2.857507,-2.85158 0,-1.56839 1.28588,-2.85169 2.857507,-2.85169 1.571616,0 2.857496,1.2833 2.857496,2.85169 0,1.56837 -1.28588,2.85158 -2.857496,2.85158 z M 51.011661,237.46873 H 16.721662 v 11.40645 h 34.289999 z\"       id=\"path4\"       style=\"stroke-width:0.26458335\" />  </g></svg>";