static const char svg_tb_map_zoom_full[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_map_zoom_full.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"2.2158129\"     inkscape:cx=\"24.771427\"     inkscape:cy=\"100.67014\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer8\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title />      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer8\"     inkscape:label=\"Icon\"     style=\"display:inline\">    <path       style=\"display:inline;fill:#00c6ff;fill-opacity:1;stroke:#000000;stroke-width:2.64583325;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       d=\"m 34.020485,30.927687 c 0,0 19.522845,10.122454 26.090404,13.70194 5.133827,2.798066 -0.0597,5.761382 -0.0597,5.761382 -3.940424,1.791105 -25.194855,13.373574 -26.150107,13.373574 -0.955254,0 -24.7978265,-12.674912 -25.9112971,-13.254166 C 2.7093605,47.763418 6.7658597,45.107255 6.7658597,45.107255 12.78676,41.807997 33.005525,30.867983 34.020485,30.927687 Z\"       id=\"path9697-9\"       inkscape:connector-curvature=\"0\"       sodipodi:nodetypes=\"cscsscc\" />    <path       style=\"display:inline;fill:#00c6ff;fill-opacity:1;stroke:#000000;stroke-width:2.64583325;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       d=\"m 34.020485,18.756855 c 0,0 19.522845,10.122455 26.090404,13.701941 5.133827,2.798065 -0.0597,5.761381 -0.0597,5.761381 C 56.110765,40.011281 34.856334,51.59375 33.901079,51.59375 32.945828,51.59375 9.1032555,38.918839 7.9897849,38.339585 2.7093601,35.592586 6.7658601,32.936424 6.7658601,32.936424 12.78676,29.637166 33.005525,18.697151 34.020485,18.756855 Z\"       id=\"path9697-8\"       inkscape:connector-curvature=\"0\"       sodipodi:nodetypes=\"cscsscc\" />    <g       style=\"display:inline\"       id=\"g839\">      <path         inkscape:connector-curvature=\"0\"         id=\"path838-8-6\"         d=\"M 33.790075,6.5689045 V 32.487225\"         style=\"display:inline;fill:none;stroke:#000084;stroke-width:7.9375;stroke-linecap:round;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <path         inkscape:connector-curvature=\"0\"         id=\"path855-4-8\"         d=\"m 33.790077,32.503243 -9.011876,-9.20633\"         style=\"fill:none;stroke:#000084;stroke-width:7.9375;stroke-linecap:round;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <path         inkscape:connector-curvature=\"0\"         id=\"path855-8-3-8\"         d=\"m 42.801951,23.296913 -9.011874,9.20633\"         style=\"display:inline;fill:none;stroke:#000084;stroke-width:7.9375;stroke-linecap:round;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />    </g>  </g></svg>";