static const char svg_tb_map3d_stereo[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_map3d_stereo.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"2.6416936\"     inkscape:cx=\"141.3907\"     inkscape:cy=\"135.46279\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <g       id=\"g1011\"       transform=\"translate(-2.0312502e-8,1.5875)\">      <g         transform=\"translate(2.7912569,2.9468401)\"         id=\"g938-2\"         style=\"display:inline;fill:#848484;fill-opacity:1\">        <path           style=\"opacity:1;fill:#848484;fill-opacity:1;stroke:none;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"           d=\"M 26.841997,273.94481 H 5.1462431 v -28.575 H 32.133742 v 21.28269\"           id=\"rect918-0\"           inkscape:connector-curvature=\"0\"           sodipodi:nodetypes=\"ccccc\" />        <path           style=\"display:inline;opacity:1;fill:#848484;fill-opacity:1;stroke:none;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"           d=\"m 37.425486,273.94481 h 21.695753 v -28.575 H 32.133742 v 21.28269\"           id=\"rect918-9-6\"           inkscape:connector-curvature=\"0\"           sodipodi:nodetypes=\"ccccc\" />      </g>      <path         sodipodi:nodetypes=\"ccccc\"         inkscape:connector-curvature=\"0\"         id=\"rect918\"         d=\"M 26.987421,274.24582 H 5.2916667 v -28.575 H 32.279166 v 21.28269\"         style=\"opacity:1;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <path         sodipodi:nodetypes=\"ccccc\"         inkscape:connector-curvature=\"0\"         id=\"rect918-9\"         d=\"m 37.57091,274.24582 h 21.695753 v -28.575 H 32.279166 v 21.28269\"         style=\"display:inline;opacity:1;fill:#ffffff;fill-opacity:1;stroke:none;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <ellipse         ry=\"8.9958334\"         rx=\"11.112499\"         cy=\"257.84164\"         cx=\"18.785416\"         id=\"path961\"         style=\"display:inline;opacity:1;fill:#ff0000;fill-opacity:1;stroke:none;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <ellipse         ry=\"8.9958334\"         rx=\"11.112499\"         cy=\"257.84164\"         cx=\"45.772915\"         id=\"path961-8\"         style=\"display:inline;opacity:1;fill:#00ff00;fill-opacity:1;stroke:none;stroke-width:2.64583325;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />    </g>  </g></svg>";