static const char svg_nb_active_legend[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"nb_active_legend.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"2.0656566\"     inkscape:cx=\"88.331443\"     inkscape:cy=\"112.06357\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <rect       style=\"fill:#0000ff;fill-opacity:1;stroke:none;stroke-width:0.75266367;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect6661\"       width=\"42.333332\"       height=\"16.933332\"       x=\"12.7\"       y=\"275.83331\" />    <rect       style=\"display:inline;fill:#00ff00;fill-opacity:1;stroke:none;stroke-width:0.75266367;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect6661-6\"       width=\"42.333332\"       height=\"16.933332\"       x=\"12.7\"       y=\"254.66666\" />    <rect       style=\"display:inline;fill:#ff0000;fill-opacity:1;stroke:none;stroke-width:0.75266367;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       id=\"rect6661-6-5\"       width=\"42.333332\"       height=\"16.933332\"       x=\"12.7\"       y=\"233.49998\" />  </g></svg>";