static const char svg_tb_map3d_rotate_z_more[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_map3d_rotate_z_more.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"1.2196745\"     inkscape:cx=\"111.09045\"     inkscape:cy=\"279.64708\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     transform=\"translate(0,-229.26665)\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <g       id=\"g850\"       transform=\"matrix(0,-0.89007569,-0.89297979,0,266.81447,292.81101)\"       style=\"stroke-width:1.12167156\">      <path         sodipodi:nodetypes=\"cssssc\"         inkscape:connector-curvature=\"0\"         id=\"path899\"         d=\"m 35.088195,232.61986 c 1.499083,-0.46301 3.058316,-0.70733 4.659051,-0.70733 11.690025,0 21.166663,13.03037 21.166666,29.10416 10e-7,16.07379 -9.476639,29.10417 -21.166666,29.10417 -11.690027,0 -21.166669,-13.03038 -21.166666,-29.10417 10e-7,-6.38708 1.496316,-12.29362 4.03324,-17.09303\"         style=\"opacity:1;fill:none;fill-opacity:1;stroke:#000084;stroke-width:5.93551159;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <g         transform=\"matrix(1.8654831,0,0,1.9100757,0.49033613,-224.18938)\"         id=\"g973\"         style=\"opacity:1;fill:#000084;fill-opacity:1;stroke:#000084;stroke-width:3.14439607;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\">        <path           style=\"opacity:1;fill:#000084;fill-opacity:1;stroke:#000084;stroke-width:3.14439607;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"           d=\"m 2.8900648,253.79903 9.2572482,-9.25725 -0.01139,9.25626 z\"           id=\"path1535\"           inkscape:connector-curvature=\"0\"           sodipodi:nodetypes=\"cccc\" />        <path           style=\"display:inline;fill:#000084;fill-opacity:1;stroke:#000084;stroke-width:3.14439607;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"           d=\"m 21.381781,253.79903 -9.257248,-9.25725 0.01139,9.25626 z\"           id=\"path1535-2\"           inkscape:connector-curvature=\"0\"           sodipodi:nodetypes=\"cccc\" />      </g>      <path         sodipodi:nodetypes=\"cc\"         inkscape:connector-curvature=\"0\"         id=\"path899-8\"         d=\"m 27.873147,236.91995 c 0.384728,-0.35913 0.778508,-0.70129 1.180774,-1.02571\"         style=\"display:inline;fill:none;fill-opacity:1;stroke:#c6c6c6;stroke-width:5.93551159;stroke-linecap:round;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <path         sodipodi:nodetypes=\"cccccccsc\"         inkscape:connector-curvature=\"0\"         d=\"m 39.246905,291.46735 11.537407,-4.62423 6.53899,-12.92574 m -18.076397,17.54997 c -4.832912,0 -25.540072,-4.22978 -25.441462,-36.47535 l 8.931267,1.90533 5.860281,1.25019 c 0,3.8458 -1.792391,27.36753 10.945744,27.51228 5.52182,0.0627 11.079892,-0.92089 17.780567,-11.74242\"         style=\"fill:#000084;fill-opacity:1;stroke:#000084;stroke-width:2.96775603;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"         id=\"path968\" />    </g>  </g></svg>";