static const char svg_tb_map_zoom_next[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_map_zoom_next.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"2.1413613\"     inkscape:cx=\"39.403433\"     inkscape:cy=\"139.54809\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer8\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title />      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer8\"     inkscape:label=\"Icon\"     style=\"display:inline\">    <path       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#000000;stroke-width:2.64583325;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       d=\"m 34.020485,30.927687 c 0,0 19.522843,10.122454 26.090403,13.701939 5.13383,2.798067 -0.0597,5.761384 -0.0597,5.761384 -3.94042,1.791105 -25.194855,13.373574 -26.150107,13.373574 -0.955254,0 -24.797825,-12.674912 -25.911296,-13.254167 -5.280424,-2.747 -1.223925,-5.403162 -1.223925,-5.403162 6.0209,-3.299258 26.239665,-14.239272 27.254625,-14.179568 z\"       id=\"path9697-9-8\"       inkscape:connector-curvature=\"0\"       sodipodi:nodetypes=\"cscsscc\" />    <path       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#000000;stroke-width:2.64583325;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       d=\"m 34.020485,18.756856 c 0,0 19.522843,10.122454 26.090403,13.70194 5.13383,2.798065 -0.0597,5.761381 -0.0597,5.761381 -3.94042,1.791104 -25.194854,13.373574 -26.150109,13.373574 -0.955251,0 -24.797822,-12.674912 -25.911293,-13.254166 C 2.709361,35.592586 6.765861,32.936424 6.765861,32.936424 12.78676,29.637166 33.005525,18.697152 34.020485,18.756856 Z\"       id=\"path9697-8-4\"       inkscape:connector-curvature=\"0\"       sodipodi:nodetypes=\"cscsscc\" />    <g       id=\"g840\"       transform=\"translate(0.01884158,1.3096966)\">      <path         inkscape:connector-curvature=\"0\"         id=\"path838\"         d=\"M 18.02935,14.245018 H 49.711964\"         style=\"display:inline;fill:none;stroke:#000084;stroke-width:7.9375;stroke-linecap:round;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <path         inkscape:connector-curvature=\"0\"         id=\"path855\"         d=\"m 49.707974,14.245018 -9.20633,9.011876\"         style=\"fill:none;stroke:#000084;stroke-width:7.9375;stroke-linecap:round;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <path         inkscape:connector-curvature=\"0\"         id=\"path855-8\"         d=\"m 40.501644,5.233144 9.20633,9.011874\"         style=\"display:inline;fill:none;stroke:#000084;stroke-width:7.9375;stroke-linecap:round;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />    </g>  </g></svg>";