static const char svg_tb_edit_shape_add[] = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><!-- Created with Inkscape (http://www.inkscape.org/) --><svg   xmlns:dc=\"http://purl.org/dc/elements/1.1/\"   xmlns:cc=\"http://creativecommons.org/ns#\"   xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"   xmlns:svg=\"http://www.w3.org/2000/svg\"   xmlns=\"http://www.w3.org/2000/svg\"   xmlns:sodipodi=\"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd\"   xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\"   width=\"256\"   height=\"256\"   viewBox=\"0 0 67.733332 67.733335\"   version=\"1.1\"   id=\"svg8\"   sodipodi:docname=\"tb_edit_shape_add.svg\"   inkscape:version=\"0.92.3 (2405546, 2018-03-11)\"   enable-background=\"new\">  <defs     id=\"defs2\" />  <sodipodi:namedview     id=\"base\"     pagecolor=\"#ffffff\"     bordercolor=\"#666666\"     borderopacity=\"1.0\"     inkscape:pageopacity=\"0.0\"     inkscape:pageshadow=\"2\"     inkscape:zoom=\"1.5766128\"     inkscape:cx=\"86.353367\"     inkscape:cy=\"110.73995\"     inkscape:document-units=\"px\"     inkscape:current-layer=\"layer3\"     showgrid=\"true\"     inkscape:window-width=\"1920\"     inkscape:window-height=\"1011\"     inkscape:window-x=\"0\"     inkscape:window-y=\"0\"     inkscape:window-maximized=\"1\"     units=\"px\"     fit-margin-top=\"-0.3\"     inkscape:snap-grids=\"false\"     inkscape:showpageshadow=\"false\"     guidetolerance=\"10000\">    <inkscape:grid       type=\"xygrid\"       id=\"grid815\"       originy=\"67.733334\" />  </sodipodi:namedview>  <metadata     id=\"metadata5\">    <rdf:RDF>      <cc:Work         rdf:about=\"\">        <dc:format>image/svg+xml</dc:format>        <dc:type           rdf:resource=\"http://purl.org/dc/dcmitype/StillImage\" />        <dc:title></dc:title>      </cc:Work>    </rdf:RDF>  </metadata>  <g     inkscape:groupmode=\"layer\"     id=\"layer3\"     inkscape:label=\"Icon\"     style=\"display:inline\"     sodipodi:insensitive=\"true\">    <path       style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#000000;stroke-width:2.64583325;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"       d=\"m 33.755901,11.348522 c 0,0 19.522849,10.122454 26.090409,13.70194 5.13382,2.798066 -0.0597,5.761382 -0.0597,5.761382 -3.94043,1.791104 -25.19486,13.373573 -26.150115,13.373573 -0.955251,0 -24.7978228,-12.674911 -25.9112937,-13.254165 C 2.4447773,28.184253 6.5012763,25.52809 6.5012763,25.52809 12.522176,22.228831 32.740941,11.288818 33.755901,11.348522 Z\"       id=\"path9697\"       inkscape:connector-curvature=\"0\"       sodipodi:nodetypes=\"cscsscc\" />    <g       id=\"g1092\"       transform=\"translate(0.26458375,-6.0854162)\">      <path         inkscape:connector-curvature=\"0\"         id=\"path2072\"         d=\"m 48.683333,33.602083 v 30.1625\"         style=\"fill:#ffffff;fill-opacity:1;stroke:#000084;stroke-width:10.58333302;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />      <g         id=\"g1084\">        <path           style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#000084;stroke-width:10.58333302;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\"           d=\"m 33.602083,48.683333 h 30.1625\"           id=\"path2072-8\"           inkscape:connector-curvature=\"0\" />        <g           id=\"g1070\">          <path             inkscape:connector-curvature=\"0\"             id=\"path2072-7\"             d=\"M 48.683332,35.6967 V 61.669966\"             style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#00c6ff;stroke-width:5.29166651;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />          <path             inkscape:connector-curvature=\"0\"             id=\"path2072-8-8\"             d=\"M 35.6967,48.683333 H 61.669966\"             style=\"display:inline;fill:#ffffff;fill-opacity:1;stroke:#00c6ff;stroke-width:5.29166651;stroke-linecap:butt;stroke-linejoin:round;stroke-miterlimit:4;stroke-dasharray:none;stroke-opacity:1\" />        </g>      </g>    </g>  </g></svg>";